@@ -169,7 +169,7 @@ KSaneViewer::~KSaneViewer()
 // ------------------------------------------------------------------------
 void KSaneViewer::setQImage(QImage *img)
 {
-    if (img == 0) {
+    if (img == nullptr) {
         return;
     }
 
@@ -511,6 +511,7 @@ void KSaneViewer::clearSelections()
 {
     clearActiveSelection();
     clearSavedSelections();
+    updateSelVisibility();
 }
 
 // ------------------------------------------------------------------------
@@ -928,6 +929,7 @@ void KSaneViewer::findSelections(float area)
                             }
                             wSelStart = -1;
                             wSelEnd = -1;
+                            (void)hSelEnd;    // clang static analyzer report hSelEnd is never used.
                             wSelMargin = 0;
                         }
                     }
