@@ -44,7 +44,9 @@
 #include <QIcon>
 
 #include <kpassworddialog.h>
+#ifdef HAVE_KF5WALLET
 #include <kwallet.h>
+#endif
 
 #include "ksaneoption.h"
 #include "ksaneoptbutton.h"
@@ -314,11 +316,13 @@ bool KSaneWidget::openDevice(const QString &deviceName)
     SANE_Word                      numSaneOptions;
     SANE_Int                       res;
     KPasswordDialog               *dlg;
+#ifdef HAVE_KF5WALLET
     KWallet::Wallet               *saneWallet;
+#endif
     QString                        myFolderName = QStringLiteral("ksane");
     QMap<QString, QString>         wallet_entry;
 
-    if (d->m_saneHandle != 0) {
+    if (d->m_saneHandle != nullptr) {
         // this KSaneWidget already has an open device
         return false;
     }
@@ -337,6 +341,7 @@ bool KSaneWidget::openDevice(const QString &deviceName)
 
     // prepare wallet for authentication and create password dialog
     if (status == SANE_STATUS_ACCESS_DENIED) {
+#ifdef HAVE_KF5WALLET
         saneWallet = KWallet::Wallet::openWallet(KWallet::Wallet::LocalWallet(), winId());
 
         if (saneWallet) {
@@ -351,7 +356,9 @@ bool KSaneWidget::openDevice(const QString &deviceName)
                 dlg->setPassword(wallet_entry[QStringLiteral("password")]);
                 dlg->setKeepPassword(true);
             }
-        } else {
+        } else
+#endif
+        {
             dlg = new KPasswordDialog(this, KPasswordDialog::ShowUsernameLine);
         }
         dlg->setPrompt(i18n("Authentication required for resource: %1", deviceName));
@@ -374,6 +381,7 @@ bool KSaneWidget::openDevice(const QString &deviceName)
 
         status = sane_open(deviceName.toLatin1().constData(), &d->m_saneHandle);
 
+#ifdef HAVE_KF5WALLET
         // store password in wallet on successful authentication
         if (dlg->keepPassword() && status != SANE_STATUS_ACCESS_DENIED) {
             QMap<QString, QString> entry;
@@ -383,6 +391,7 @@ bool KSaneWidget::openDevice(const QString &deviceName)
                 saneWallet->writeMap(deviceName, entry);
             }
         }
+#endif
     }
 
     if (status != SANE_STATUS_GOOD) {
@@ -406,7 +415,7 @@ bool KSaneWidget::openDevice(const QString &deviceName)
 
     // Read the options (start with option 0 the number of parameters)
     optDesc = sane_get_option_descriptor(d->m_saneHandle, 0);
-    if (optDesc == 0) {
+    if (optDesc == nullptr) {
         d->m_auth->clearDeviceAuth(d->m_devName);
         d->m_devName.clear();
         return false;
@@ -520,7 +529,7 @@ bool KSaneWidget::closeDevice()
     d->m_auth->clearDeviceAuth(d->m_devName);
     // else
     sane_close(d->m_saneHandle);
-    d->m_saneHandle = 0;
+    d->m_saneHandle = nullptr;
     d->clearDeviceOptions();
 
     // disable the interface until a new device is opened.
@@ -676,7 +685,7 @@ bool KSaneWidget::getOptVal(const QString &optname, QString &value)
 {
     KSaneOption *option;
 
-    if ((option = d->getOption(optname)) != 0) {
+    if ((option = d->getOption(optname)) != nullptr) {
         return option->getValue(value);
     }
     // Special handling for non sane option
@@ -739,7 +748,7 @@ bool KSaneWidget::setOptVal(const QString &option, const QString &value)
 {
     KSaneOption *opt;
 
-    if ((opt = d->getOption(option)) != 0) {
+    if ((opt = d->getOption(option)) != nullptr) {
         if (opt->setValue(value)) {
             if ((d->m_splitGamChB) &&
                     (d->m_optGamR) &&
@@ -784,7 +793,7 @@ bool KSaneWidget::setOptVal(const QString &option, const QString &value)
 
 void KSaneWidget::setScanButtonText(const QString &scanLabel)
 {
-    if (d->m_scanBtn == 0) {
+    if (d->m_scanBtn == nullptr) {
         qCritical() << "setScanButtonText was called before KSaneWidget was initialized";
         return;
     }
@@ -793,7 +802,7 @@ void KSaneWidget::setScanButtonText(const QString &scanLabel)
 
 void KSaneWidget::setPreviewButtonText(const QString &previewLabel)
 {
-    if (d->m_scanBtn == 0) {
+    if (d->m_scanBtn == nullptr) {
         qCritical() << "setPreviewButtonText was called before KSaneWidget was initialized";
         return;
     }
